diff --git a/src/mmg3d/analys_3d.c b/src/mmg3d/analys_3d.c
index c2659359..3a08d555 100644
--- a/src/mmg3d/analys_3d.c
+++ b/src/mmg3d/analys_3d.c
@@ -772,6 +772,85 @@ int MMG3D_nmgeom(MMG5_pMesh mesh){
   return 1;
 }
 
+/**
+ * \param mesh pointer toward mesh structure
+ * \return 1 if success, 0 if fail
+ *
+ * Assign surface references demending on geometric features: portion of
+ * surfaces shared by non-manifold or ridge edges will have different
+ * references.
+ */
+int MMG3D_Set_faceColors(MMG5_pMesh mesh) {
+  MMG5_pTria   pt,pt1;
+  int          *adja,*pile,ipil;
+  int          k,kk,ncc=0;
+  int8_t       i;
+
+  MMG5_SAFE_MALLOC(pile,mesh->nt+1,int,return 0);
+
+  /* Reset flags */
+  for (kk=1; kk<=mesh->nt; kk++) {
+    mesh->tria[kk].flag = 0;
+  }
+
+  pile[1] = 1;
+  ipil    = 1;
+  while ( ipil > 0 ) {
+    ncc++;
+
+    /* Pile up triangles that can be seen from the initial one without crossing
+     * a ridge or non manifold edge. Change surface reference of stacked
+     * triangles */
+    do {
+      k  = pile[ipil--];
+      pt = &mesh->tria[k];
+      pt->flag = ncc;
+
+      /* Assign new reference to portion of surface */
+      pt->ref  = ncc;
+
+      if ( !MG_EOK(pt) )  continue;
+
+      adja = &mesh->adjt[3*(k-1)+1];
+      for (i=0; i<3; i++) {
+        /* Don't cross ridges or non-manifold edges */
+        if ( (pt->tag[i] & MG_GEO) || (pt->tag[i] & MG_NOM) ) continue;
+
+        /* store adjacent */
+        kk = adja[i] / 3;
+        pt1 = &mesh->tria[kk];
+        if ( !pt1->flag ) {
+          pt1->flag    = ncc;
+          pile[++ipil] = kk;
+        }
+      }
+    }
+    while ( ipil > 0 );
+
+    /* find next unmarked triangle */
+    ipil = 0;
+    for (kk=1; kk<=mesh->nt; kk++) {
+      pt = &mesh->tria[kk];
+      if ( MG_EOK(pt) && (pt->flag == 0) ) {
+        ipil = 1;
+        pile[ipil] = kk;
+        pt->flag   = ncc+1;
+        break;
+      }
+    }
+  }
+
+  /* bilan */
+  if ( abs(mesh->info.imprim) > 3 || mesh->info.ddebug ) {
+    fprintf(stdout,"     %d surface area detected: assign new references\n",ncc);
+  }
+
+  MMG5_SAFE_FREE(pile);
+  return 1;
+}
+
+
+
 /** preprocessing stage: mesh analysis */
 int MMG3D_analys(MMG5_pMesh mesh) {
   MMG5_Hash hash;
@@ -875,6 +954,10 @@ int MMG3D_analys(MMG5_pMesh mesh) {
     return 0;
   }
 
+  if ( getenv("MMG_COLOR_SURFAREA") ) {
+    if ( !MMG3D_Set_faceColors(mesh) ) return 0;
+  }
+
   /* set bdry entities to tetra and fill the orientation field */
   if ( !MMG5_bdrySet(mesh) ) {
     fprintf(stderr,"\n  ## Boundary problem. Exit program.\n");
